class MidpointOdeSolver {
  PVector[] midPoints;
  PVector[] accels;

  MidpointOdeSolver(int nPoints) {
    midPoints = new PVector[nPoints];
    accels = new PVector[nPoints];
    for(int i = 0; i < midPoints.length; i++) {
      midPoints[i] = new PVector();
      accels[i] = new PVector();
    }
  }

  void solve(Point[] points, Link[] links, float delta) {
    
    // half update position
    for(int i = 0; i < points.length; i++) {
      midPoints[i] = PVector.add(points[i].p, PVector.mult(points[i].v, delta / 2)); 
    }
    
    // apply forces
    for(int i = 0; i < points.length; i++) {
      // reset
      accels[i].set(0, 0);

      // gravity
      accels[i].add(0, 100);
    }
    
    // apply forces caused by links
    for(int i = 0; i < links.length; i++) {
      // spring force
      // Fk = -(l - l0) * k * dir / |dir|
      PVector dir = PVector.sub(midPoints[links[i].p1], midPoints[links[i].p0]);
      float abs_dir = dir.mag();
      PVector unit_dir = PVector.div(dir, abs_dir);

      //PVector f_k = PVector.div(PVector.mult(dir, (abs_dir - links[i].l) * links[i].k), abs_dir);
      PVector f_k = PVector.mult(unit_dir, (abs_dir - links[i].l) * links[i].k);
      accels[links[i].p0].add(f_k.x / points[links[i].p0].m, f_k.y / points[links[i].p0].m);
      accels[links[i].p1].add(-f_k.x / points[links[i].p1].m, -f_k.y / points[links[i].p1].m);

      // viscos force
      // Fc = - (v dot unit_dir) * c * unit_dir
      float dir_v0 = PVector.dot(points[links[i].p0].v, unit_dir);
      PVector f_c0 = PVector.mult(unit_dir, -dir_v0 * links[i].c);
      float dir_v1 = PVector.dot(points[links[i].p1].v, unit_dir);
      PVector f_c1 = PVector.mult(unit_dir, -dir_v1 * links[i].c);
      accels[links[i].p0].add(f_c0.x / points[links[i].p0].m, f_c0.y / points[links[i].p0].m);
      accels[links[i].p1].add(f_c1.x / points[links[i].p1].m, f_c1.y / points[links[i].p1].m);
  
}

    // update position / velocity
    for(int i = 0; i < points.length; i++) {

      if(!points[i].fixed) {
        // half update velocity
        PVector vMid = PVector.add(points[i].v, PVector.mult(accels[i], delta / 2));
  
        // update position
        points[i].p.add(PVector.mult(vMid, delta));
        
        // update velocity
        points[i].v.add(PVector.mult(accels[i], delta));
      }
    }

  }

}
